#! /bin/sh
#
# autogen.sh - Shellscript para ejecutar Autotools

package=solvelen
DEBUG=
BUILDDIR=$(dirname $0)
abs_BUILDDIR=$(pwd)
CONFIG_AC_FILE="$BUILDDIR/configure.ac"

# Imprime mensaje si DEBUG esta definido
debug() {
    if [ ! -z "$DEBUG" ]
    then
	echo "DEBUG: $1"
    fi
}

# Funcion de terminacion para shell y subshell
fin() {
    (exit $1)
    exit $1
}

# Funcion para actualizar fecha en archivos po
poupdate() {
    rm $1
    mv ../$1 $1
    tpo=`cat $1 | grep 'PO-Revision-Date:' | sed -e s/\"//g | sed -e s'/.*: \(.*\)[\]n/\1/'`
    echo "Fecha ultima revision: ${tpo}"
}

# Mensaje paquete ausente
#   msj_pkg_absent 'nombre paquete' 'url descarga paquete'
msj_pkg_absent() {
    echo "The package $1 is not installed actually."
    echo "You can install the package according to your"
    echo "distribution or download the source code at:"
    echo "<$2>"
    fin 1
}

# Mensaje paquete antiguo
#   msj_pkg_old 'nombre paquete' 'version' 'url descarga paquete'
msj_pkg_old() {
    echo "To compile '$package' is necesary have $1 $2"
    echo "or later. Get the latest version from:"
    echo "<$3>"
    fin 1
}

# Verificacion version de paquete Autotool
#   check_version 'nombre paquete (ejecutable)' 'url descarga paquete' '# mayor' '# menor' '# micro'
check_version() {
    PACKAGE=$1
    URL=$2
    MAYOR=$3
    MENOR=$4
    MICRO=$5
    debug "version requerida: mayor $MAYOR menor $MENOR micro $MICRO"
    VERSION=$MAYOR
    if [ ! -z "$MENOR" ]; then VERSION=$VERSION.$MENOR; else MENOR=0; fi
    if [ ! -z "$MICRO" ]; then VERSION=$VERSION.$MICRO; else MICRO=0; fi
    echo ": checking $PACKAGE with version $VERSION..."
    ($PACKAGE --version) < /dev/null > /dev/null 2>&1 || msj_pkg_absent $PACKAGE $URL
    pkg_version_ins=`$PACKAGE --version | head -n 1 | sed -e 's/\.exe//g' -e 's/[^0-9\.]*//g'`
    pkg_mayor=`echo $pkg_version_ins | cut -d. -f1`
    pkg_menor=`echo $pkg_version_ins | cut -d. -f2`
    pkg_micro=`echo $pkg_version_ins | cut -d. -f3`
    if [ -z "$pkg_menor" ]; then
	pkg_menor=0
    fi
    if [ -z "$pkg_micro" ]; then
	pkg_micro=0
    fi
    debug "version hallada: mayor $pkg_mayor menor $pkg_menor micro $pkg_micro"
    TESTPKG=
    if [ "$pkg_mayor" -lt "$MAYOR" ]; then
	TESTPKG=1
    elif [ "$pkg_mayor" -eq "$MAYOR" ]; then
	if [ "$pkg_menor" -lt "$MENOR" ]; then
	    TESTPKG=1
	elif [ "$pkg_menor" -eq "$MENOR" ]; then
	    if [ "$pkg_micro" -lt "$MICRO" ]; then
		TESTPKG=1
	    fi
	fi
    fi
    if [ ! -z "$TESTPKG" ]; then
	echo ": package $PACKAGE with version $pkg_version_ins... outdated"
	msj_pkg_old $PACKAGE $VERSION $URL
    else
	echo ": package $PACKAGE with version $pkg_version_ins... ok"
    fi
}

check_version "autoconf" "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 67 || fin 1
check_version "automake" "ftp://ftp.gnu.org/pub/gnu/automake/" 1 11 1 || fin 1
check_version "libtool" "ftp://ftp.gnu.org/pub/gnu/libtool/" 2 4 || fin 1
check_version "gettext" "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 17 || fin 1

# Directorio para correr autotools
if [ ! -f "$CONFIG_AC_FILE" ]; then
  echo "Are you sure $BUILDDIR is a valid source directory?"
  fin 1
fi

# Ejecutar Autotools
mkdir -p m4
echo "running autoreconf..."
autoreconf -i
echo "autoreconf OK!"

# Crer archivos gettext
echo "Creating file Makevars"
sed -e 's/^COPYRIGHT_HOLDER.*/COPYRIGHT_HOLDER = Miguel Molina/' \
    -e 's/^MSGID_BUGS_ADDRESS.*/MSGID_BUGS_ADDRESS = $(PACKAGE BUGREPORT)/' \
    po/Makevars.template > po/Makevars
echo "Creating initial po/POTFILES.in"
echo "# List of source files which contain translatable strings" > po/POTFILES.in
echo "src/mainSLEn.cpp" >> po/POTFILES.in
echo "src/biseccion.cpp" >> po/POTFILES.in

# Pre-construccion
./configure --prefix=`pwd`/tmp
make
echo "Creating po template..."
file="solvelen.pot"
if [ "-f $file" ] && [ -f "po/$file" ]
then
   rm -f po/$file
   mv $file po/$file
else
   echo "No such: $file"
   fin 1
fi

# i18n - l10n
echo "Creating po-files..."
langlist="es pt_BR la"
cd po
for lang in $langlist
do
   file="../$lang.po"
   if [ -f "$file" ]
   then
      msginit --no-translator -l $lang
      poupdate $lang.po
      msgfmt --statistics --check $lang.po
      echo $lang >> LINGUAS
   else
      echo "No such: $file"
      fin 1
   fi
done
cd ..

# Construccion del paquete
make maintainer-clean
./configure --prefix=`pwd`/tmp
make distcheck

echo "autogen OK!"

fin 0
