/*
 * LaneEmden.cpp - Funciones de la clase LaneEmden 
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file LaneEmden.cpp
 * @brief Implementación de la clase LaneEmden
 */

#include"LaneEmden.h"
#include<cmath>


/* Matriz de Butcher para RKS */
void LaneEmden::MatrizButcher(void){
  int i, j;
  for(i=0;i<S;i++)
    for(j=0;j<S;j++)
      A[i][j]=0.0;
  // Metodo Runge-Kutta orden 6
  double l=sqrt(21.0);
  A[1][0]=1.0;
  A[2][0]=3.0/8.0;  A[2][1]=1.0/8.0;
  A[3][0]=8.0/27.0;  A[3][1]=2.0/27.0;  A[3][2]=8.0/27.0;
  A[4][0]=-3.0*(7.0-3.0*l)/392.0;  A[4][1]=-8.0*(7.0-l)/392.0;  A[4][2]=48.0*(7.0-l)/392.0;  A[4][3]=-3.0*(21.0-l)/392.0;
  A[5][0]=-5.0*(231.0+51.0*l)/1960.0;  A[5][1]=-40.0*(7.0+l)/1960.0;  A[5][2]=-320.0*l/1960.0;  A[5][3]=3.0*(21.0+121.0*l)/1960.0;
  A[5][4]=392.0*(6.0+l)/1960.0;
  A[6][0]=15.0*(22.0+7.0*l)/180.0;  A[6][1]=120.0/180.0;  A[6][2]=-40.0*(5.0-7.0*l)/180.0;  A[6][3]=63.0*(2.0-3.0*l)/180.0;
  A[6][4]=-14.0*(49.0+9.0*l)/180.0;  A[6][5]=70.0*(7.0-l)/180.0;
  B[0]=9.0/180.0;  B[1]=0.0;  B[2]=64.0/180.0;  B[3]=0.0;      B[4]=49.0/180.0;    B[5]=49.0/180.0;    B[6]=9.0/180.0;
  C[0]=0.0;        C[1]=1.0;  C[2]=1.0/2.0;     C[3]=2.0/3.0;  C[4]=(7.0-l)/14.0;  C[5]=(7.0+l)/14.0;  C[6]=1.0;
  /*
  // Verificacion
  double suma;
  for(i=1;i<S;i++){
    suma=0.0;
    for(j=0;j<i;j++)
      suma+=A[i][j];
    cout << "r[" << i << "]=" << suma-C[i] << endl;
  }
  */
}

/* Metodo Runge-Kutta orden S */
void LaneEmden::RungeKuttaS(double h){
  int i, j, k;
  double taux, suma;
  double Ki[S][N], Raux[N], Ktmp[N];
  // Calculo de K1
  Ecuacion(t,R,Ktmp);
  for(j=0;j<N;j++)
    Ki[0][j]=Ktmp[j];
  // Calculo de Ki
  for(i=1;i<S;i++){
    taux=t+h*C[i];
    for(j=0;j<N;j++){
      suma=0.0;
      for(k=0;k<=i-1;k++)
	suma+=A[i][k]*Ki[k][j];
      Raux[j]=R[j]+h*suma;
    }
    Ecuacion(taux,Raux,Ktmp);
    for(j=0;j<N;j++)
      Ki[i][j]=Ktmp[j];
  }
  // Calculo de Rpos
  for(j=0;j<N;j++){
    suma=0.0;
    for(i=0;i<S;i++)
      suma+=B[i]*Ki[i][j];
    R[j]+=h*suma;
  }
}

/* Ecuacion Lane-Emden, N=2 */
void LaneEmden::Ecuacion(double tn,double *Rn,double *Fn){
  double xn, yn;
  xn=Rn[0];  yn=Rn[1];
  // Bandera para xn negativo
  if( xn < 0.0 )
    sgn=0;
  // En t=0, usar L'Hopital
  Fn[0]=yn;
  if( tn==0 )
    Fn[1]=-1.0/3.0;
  else
    Fn[1]=-2.0*yn/tn-pow(xn,n);
}

/* Condiciones iniciales */
void LaneEmden::inicio(double n0){
  R[0]=1.0;  R[1]=0.0;  n=n0;  sgn=1;
}

/* Funcion auxiliar para actualizar Rn */
void LaneEmden::updateRn(double tn){
  t=tn;  x=R[0];  y=R[1];
}
