/*
 * biseccion.cpp - Metodo de la biseccion para hallar la raiz de una funcion
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file biseccion.cpp
 * @brief Implementación de función para el método de la bisección
 */

#include<iostream>
#include<cstdlib>
#include<cmath>
#include"nsolveLEn.h"

/// @cond
#ifdef ENABLE_NLS
# include <locale.h>
# include "gettext.h"
# define _(string) (const char *) gettext(string)
#else
# define _(string) (string)
#endif
/// @endcond

using namespace std;

/// @cond INTERNAL

/* Metodo de la biseccion */
double biseccion(double *fx, double epsX, double epsF){
  double a, b, m, fa, fb, fm;
  // Soporte i18n
#ifdef ENABLE_NLS
  setlocale(LC_NUMERIC, "C");
  setlocale(LC_TIME, "");
  setlocale(LC_MONETARY, "");
  setlocale(LC_CTYPE, "");
  setlocale(LC_COLLATE, "");
  setlocale(LC_MESSAGES, "");
  textdomain(PACKAGE);
  bindtextdomain(PACKAGE, LOCALEDIR);
#endif /* ENABLE_NLS */
  a=fx[4]+dt;
  b=fx[4]+2.0*dt;
  fa=InterpolacionTheta(a,fx);
  fb=InterpolacionTheta(b,fx);
  if( fabs(fa) <= epsF )
    return a;
  if( fabs(fb) <= epsF )
    return b;
  if( fa*fb > 0 ){
    cerr << _("Error: bisection method; the function does not cross by the axis x in the interval [") << a << ", " << b << "]." << endl;
    exit(EXIT_FAILURE);
  }
  while( b-a > epsX ){
    m=(a+b)/2.0;
    fm=InterpolacionTheta(m,fx);
    if( fabs(fm) <= epsF )
      return m;
    // nuevo intervalo [a,m]
    if( fa*fm < 0.0 ){
      b=m;
      fb=fm;
    }
    // nuevo intervalo [m,b]
    else{
      a=m;
      fa=fm;
    }
  }
  if( fabs(fa) <= fabs(fb) )
    return a;
  else
    return b;
}

/// @endcond
