/*
 * biseccion.h - Declaracion de funcion metodo de la biseccion
 *
 *    Copyright (C) 2012 - 2015  Miguel Molina
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * @file biseccion.h
 * @brief Declaración de función 
 * @details Definición de función método de la bisección
 */

#ifndef __BISECCION_H_
#define __BISECCION_H_

extern const double dt;              /**< Variable global para el paso de integración */
/**
 * @brief Función Método de la bisección
 * @param fx  Función \f$f(x)\f$
 * @param epsX  Error en \f$\epsilon_x\f$
 * @param epsF  Error en \f$\epsilon_y\f$
 * @return Entrega raíz de la función \f$f(x)\f$.
 * @details Declaración del método de la función para hallar la raíz
 * de una función \f$f(x)\f$; la iteración termina cuando el error es
 * menor a \f$\epsilon_x\f$ y/o \f$\epsilon_y\f$.
 */
double biseccion(double *fx, double epsX, double epsF);

#endif /* __BISECCION_H_ */
